module ex ( 
	/* Inputs */
	clock, start, reset, i_x, i_y, i_u, i_dx,
	/* Outputs */
	o_x, o_y, o_u, comp
	);
	
	input clock;
	input start;
	input reset;
	input [31:0] i_x;
	input [31:0] i_y;
	input [31:0] i_u;
	input [31:0] i_dx;
	output [31:0] o_x;
	output [31:0] o_y;
	output [31:0] o_u;
	output comp;

	wire [31:0] x_out, y_out, u_out, dx_out;
	wire [31:0] add0_out, mul0_out, mul1_out, mul2_out;
	wire [31:0] st1_y_out, st1_u_out, st1_dx_out;
	wire [31:0] add0_reg_out, mul0_reg_out, mul1_reg_out, mul2_reg_out;
	wire [31:0] add1_out, mul3_out, mul4_out;
	wire [31:0] st2_u_out, st2_add0_reg_out;
	wire [31:0] add1_reg_out, mul3_reg_out, mul4_reg_out;	
	wire [31:0] sub0_out;
	wire [31:0] st3_mul4_reg_out, st3_add0_reg_out, st3_add1_reg_out;
	wire [31:0] sub0_reg_out;	
	wire [31:0] sub1_out;
		
	/* Control part */
	wire s0out, s1out, s2out, s3out;
	dff1 s0 (.in(start), .reset(reset), .clock(clock), .out(s0out));
	dff1 s1 (.in(s0out), .reset(reset), .clock(clock), .out(s1out));
	dff1 s2 (.in(s1out), .reset(reset), .clock(clock), .out(s2out));
	dff1 s3 (.in(s2out), .reset(reset), .clock(clock), .out(s3out));
	dff1 s4 (.in(s3out), .reset(reset), .clock(clock), .out(comp));	
	
	/* Datapath part */
	/* Stage 0 */
	dffe32 x  (.in(i_x), .reset(reset), .clock(clock), .en(start), .out(x_out));
	dffe32 y  (.in(i_y), .reset(reset), .clock(clock), .en(start), .out(y_out));
	dffe32 u  (.in(i_u), .reset(reset), .clock(clock), .en(start), .out(u_out));
	dffe32 dx (.in(i_dx), .reset(reset), .clock(clock), .en(start), .out(dx_out));
	
	/* Stage 1 */
	add32     add0 (.a(x_out), .b(dx_out), .out(add0_out));
	mult32_32 mul0 (.a(x_out), .b(3), .out(mul0_out));
	mult32_32 mul1 (.a(u_out), .b(dx_out), .out(mul1_out));
	mult32_32 mul2 (.a(y_out), .b(3), .out(mul2_out));
	
	dffe32 st1_y    (.in(y_out), .reset(reset), .clock(clock), .en(s0out), .out(st1_y_out));
	dffe32 st1_u    (.in(u_out), .reset(reset), .clock(clock), .en(s0out), .out(st1_u_out));
	dffe32 st1_dx   (.in(dx_out), .reset(reset), .clock(clock), .en(s0out), .out(st1_dx_out));
	dffe32 add0_reg (.in(add0_out), .reset(reset), .clock(clock), .en(s0out), .out(add0_reg_out));
	dffe32 mul0_reg (.in(mul0_out), .reset(reset), .clock(clock), .en(s0out), .out(mul0_reg_out));
	dffe32 mul1_reg (.in(mul1_out), .reset(reset), .clock(clock), .en(s0out), .out(mul1_reg_out));
	dffe32 mul2_reg (.in(mul2_out), .reset(reset), .clock(clock), .en(s0out), .out(mul2_reg_out));
	
	/* Stage 2 */
	add32     add1 (.a(st1_y_out), .b(mul1_reg_out), .out(add1_out));
	mult32_32 mul3 (.a(mul0_reg_out), .b(mul1_reg_out), .out(mul3_out));
	mult32_32 mul4 (.a(mul2_reg_out), .b(st1_dx_out), .out(mul4_out));
	
	dffe32 st2_u    (.in(st1_u_out), .reset(reset), .clock(clock), .en(s1out), .out(st2_u_out));
	dffe32 add1_reg (.in(add1_out), .reset(reset), .clock(clock), .en(s1out), .out(add1_reg_out));
	dffe32 mul3_reg (.in(mul3_out), .reset(reset), .clock(clock), .en(s1out), .out(mul3_reg_out));
	dffe32 mul4_reg (.in(mul4_out), .reset(reset), .clock(clock), .en(s1out), .out(mul4_reg_out));
	dffe32 st2_add0_reg (.in(add0_reg_out), .reset(reset), .clock(clock), .en(s1out), .out(st2_add0_reg_out));
	
	/* Stage 3 */
	sub32	sub0     (.a(st2_u_out), .b(mul3_reg_out), .out(sub0_out));
	
	dffe32 sub0_reg (.in(sub0_out), .reset(reset), .clock(clock), .en(s2out), .out(sub0_reg_out));
	dffe32 st3_mul4_reg (.in(mul4_reg_out), .reset(reset), .clock(clock), .en(s2out), .out(st3_mul4_reg_out));
	dffe32 st3_add0_reg (.in(st2_add0_reg_out), .reset(reset), .clock(clock), .en(s2out), .out(st3_add0_reg_out));	
	dffe32 st3_add1_reg (.in(add1_reg_out), .reset(reset), .clock(clock), .en(s2out), .out(st3_add1_reg_out));	
	
	/* Stage 4 */
	sub32	sub1     (.a(sub0_reg_out), .b(st3_mul4_reg_out), .out(sub1_out));
	
	dffe32 sub1_reg (.in(sub1_out), .reset(reset), .clock(clock), .en(s3out), .out(o_u));
	dffe32 st4_add0_reg (.in(st3_add0_reg_out), .reset(reset), .clock(clock), .en(s3out), .out(o_x));
	dffe32 st4_add1_reg (.in(st3_add1_reg_out), .reset(reset), .clock(clock), .en(s3out), .out(o_y));	
	
endmodule
